# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：net.py
#   版本：0.1
#   作者：lds
#   日期：2018-07-05
#   语言：Python 3.X
#   说明：网络相关的函数集合
# ---------------------------------------

import socket


def get_host_ip():
    """
    通过 UDP 获取本机 IP
    print(get_host_ip())
    :return:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_host_ip)
    # doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=dict_val_to_key)
    # doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=list_to_dict)
    # doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_kuohao_feijie)
    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------
    from time import time, sleep

    start_time = t1 = time()

    doc()

    print('运行时间 %.2f 秒' % (time() - start_time))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：spider.py
#   版本：0.3
#   作者：lds
#   日期：2018-11-26
#   语言：Python 3.X
#   说明：爬虫用到的函数集合
# ---------------------------------------



import os
# import datetime
from time import time, sleep
# random_sleep
import random
# get_response
import requests
from requests.exceptions import RequestException
# download_file
import re
from ilds.file import is_file
from urllib import parse


def random_sleep(start=1, end=3):
    """
    随机延迟，因为如果你访问了很多页面，你的ip可能会被封。
    :param start:
    :param end:
    :return:
    """
    sleep_time = random.randint(start, end)
    # sleep_time = random.randint(250, 1000) / 1000.0
    print('随机延迟：%s 秒......' % sleep_time)
    sleep(sleep_time)


def get_response(url, params=None, **kwargs):
    """
    请求获取网页内容，为了防止程序中断，捕捉错误，返回 None。
    :param url:
    :param params:
    :param kwargs:
    :return:
    """
    try:
        response = requests.get(url, params=params, **kwargs)
        if response.status_code == 200:
            return response
        return None
    except RequestException:
        return None


def get_response_to_file(url, file_name=None, params=None, **kwargs):
    """
    请求获取网页内容，并保存到文件。
    :param url:
    :param file_name:
    :param params:
    :param kwargs:
    :return:
    """

    if file_name is None:
        file_name = url.split('/')[-1]
        file_name = re.sub('[\/:*?"<>|]', '-', file_name)

    ret = get_response(url, params=params, **kwargs)
    if ret:
        with open(file_name, 'w', encoding='utf-8') as fout:  # , encoding = 'utf-8'
            fout.write(ret.text)
        return file_name
    else:
        return None


def get_file_content(file_name=None):
    """
    读取文件内容
    :param file_name:
    :return:
    """
    # 写入本地文件
    # file = '测试.html'
    # with open(file, 'w', encoding='utf-8') as fout:  # , encoding = 'utf-8'
    #     fout.write(r.text)
    if not is_file(file_name):
        return None
    with open(file_name, 'r', encoding='utf-8') as fin:
        # content = fin.read()
        return fin.read()


def download_file(url, file_name=None):
    """
    下载文件
    :param url:
    :param file_name:
    :return:
    """

    r = requests.get(url, stream=True)
    # print(type(r.headers.get('Content-Disposition')))
    content_disposition = r.headers.get('Content-Disposition')

    if file_name is None:
        if content_disposition and 'attachment; filename="' in content_disposition:
            file_name = content_disposition[22:-1]
        else:
            file_name = url.split('/')[-1]
            file_name = re.sub('[\/:*?"<>|]', '-', file_name)

    file_size = r.headers.get('Content-Length')
    if file_size is None:
        # pprint(r.headers)
        print('文件大小获取失败，%s 不是合法文件......' % file_name)
        return None

    print("开始下载：%s \n文件大小：%s" % (file_name,file_size))

    with open(file_name, 'wb') as f:
        for chunk in r.iter_content(chunk_size=1024*1024):
            if chunk:
                f.write(chunk)
    print("%s 下载完成!\n" % file_name)
    return file_name


def url_to_dict(url):
    """把url转换为dict字典"""
    params = {}
    url = parse.unquote(url)
    for param in url.split('&'):
        #print(param)
        k, v = param.split('=')
        params[k] = v
        # print("'{}': '{}',".format(k, v),)
    return params


def get_cookie_dict(cookie):
    """转换 Raw 格式的 Cookie 为字典格式"""
    params = {}
    cookie = cookie.strip()
    if cookie.startswith('Cookie:'):
        cookie = cookie[7:].strip()
    for param in cookie.split(';'):
        k, v = param.split('=')
        params[k.strip()] = v.strip()
        # print("'{}': '{}',".format(k, v), )
    return params


def doc():
    """
    打印模块说明文档
    """
    doc_text = """
路径和网址url互相转换
    from urllib.request import pathname2url
    from urllib.request import url2pathname
    str2 = parse.quote(str1)  # url 编码
    str3 = parse.unquote(str2)  # url 解码
    """
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=random_sleep)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_response)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_response_to_file)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_file_content)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=download_file)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=url_to_dict)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_cookie_dict)

    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------
    start_time = t1 = time()

    doc()

    print('运行时间 %.2f 秒' % (time() - start_time))



from colorama import Fore, Back, Style


def prompt_list(in_list,title='选择内容：',text='输入序号：'):
    """
    命令行选择列表内容的交互提示
    选择列表内容，输入 e 返回 None
    """
    print(Fore.BLUE + title)
    for i,v in enumerate(in_list):
        print(Fore.RED+f'({i}):', Fore.GREEN+str(v))
    ret = None
    while True:
        input_str = input(Fore.YELLOW+text)
        # print(input_str, type(input_str), int(input_str))
        if input_str == 'e':
            break
        try:
            ret = in_list[int(input_str)]
            break
        except:
            continue
    print(Style.RESET_ALL)
    return ret


def confirm_yes_no(title='是否确认继续...',text='输入 y 确认，其他任意字符表示取消'):
    """
    命令行选择列表内容的交互提示
    输入 y 返回 True，其他任意字符 返回 False
    """
    print(Fore.RED + title)
    print(Fore.YELLOW + text)
    input_str = input(Fore.BLUE + '请输入选择：')
    ret = False
    if input_str.lower() == 'y':
        ret = True

    print(Style.RESET_ALL)
    return ret


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=prompt_list)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=confirm_yes_no)

    print(doc_text)

if __name__ == '__main__':


    l = [1111,3333]
    ret = prompt_list(l)
    if ret == None:
        print('没有选择内容直接退出了')
    else:
        print('选择了', ret)

    ret = confirm_yes_no()
    if ret:
        print('选择了确认')
    else:
        print('选择了取消')


# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：md.py
#   版本：0.2
#   作者：lds
#   日期：2019-01-02
#   语言：Python 3.X
#   说明：处理 markdown 的函数集合
"""
20190102 html_to_md 添加按序号重命名图片文件
"""
# ---------------------------------------

import os
import re
import shutil

import chardet
import requests
import html2text

# 匹配图像链接的正则表达式 # *([^\n]+?) *#* *(?:\n+|$)   ![图像](../images/00038.jpeg)
IMG_LINK_COMPILE = re.compile(r'!\[.*?\]\((.*?)\)')


def html_to_md(html_file, save_file=None, end=False, rename_img=False, img_start_index=0):
    """
    转换 html 为 markdown
    :param html_file: 要转换的文件
    # :param save_to:
    # :param img_fun: 本来打算用来修复图片链接，但是有问题所以暂停使用
    :param save_file: 保存文件名（默认是）
    :param end:
    :param rename_img: 重命名 md 文件中的图片为序号(如：000001.jpg)
    :param img_start_index: 开始重命名图片的序号
    :return:
    """

    # html = open(html_file, 'r', encoding='utf-8').read()
    # with open(html_file, 'r', encoding='utf-8') as html:
    with open(html_file, 'rb') as fp:
        # import chardet
        # 先能识别文件的编码方式，然后根据此编码方式进行对文件编码，获取文件内容。 """
        html_data = fp.read()
        result = chardet.detect(html_data)
        file_content = html_data.decode(encoding=result['encoding'])

        text_maker = html2text.HTML2Text()
        text_maker.body_width = 0
        # text_maker.ignore_links = True # 忽略链接
        # text_maker.kypass_tables = False # 循环表
        text_maker.kypass_tables = True

        text = text_maker.handle(file_content)  # html.read()

        # 处理表格有多余空行的问题
        text = re.sub('\n\n\| \n\n', '|', text)  # 替换 | 符号的换行
        text = re.sub('\n  \n', '\n', text)  # 替换两个换行为一个
        # print(text)

        file_path, file_name = os.path.split(html_file)  # 分离 文件路径 文件名+扩展名
        # print(file_path, file_name)

        # 默认文件名
        if save_file is None:
            f_name, _ = os.path.splitext(html_file)
            save_file = f_name + ".md"
        else:
            f_name, _ = os.path.splitext(save_file)

        save_to = os.path.dirname(save_file)

        # 处理图片文件
        link_result = IMG_LINK_COMPILE.findall(text)
        if link_result:
            # print(f_line, link_result)
            # c = 0
            # continue
            for result in link_result:
                # print(result)

                # if img_fun is not None:
                #     result = img_fun(result)

                # 图片名称
                image_name = os.path.split(result)[1]
                if rename_img:
                    _ext = os.path.splitext(image_name)[1]
                    if _ext:
                        image_name = "%06d%s" % (img_start_index, _ext)
                    else:
                        image_name = "%06d.jpg" % (img_start_index)
                # 图片文件
                file = os.path.join(file_path, result)

                # 图片存放路径
                save_dir_name = os.path.split(f_name)[-1]
                save_img_path = os.path.join(save_to, 'img', save_dir_name)
                if not os.path.exists(save_img_path):
                    os.makedirs(save_img_path)

                save_img = os.path.join(save_img_path, image_name)
                # print('保存路径：', save_img_path, '图片：', save_img)

                if os.path.exists(file):
                    # print('文件已经存在', file)
                    shutil.copy(file, save_img)
                else:
                    try:
                        res = requests.get(result, verify=False)
                        with open(save_img, 'wb') as f:
                            f.write(res.content)
                        print('下载图片：', result, '保存路径：', save_img)
                    except Exception as e:
                        print('文件', file, e)

                new_image = f'./img/{save_dir_name}/{image_name}'
                text = text.replace(result, new_image)
                # print(new_image)
                img_start_index += 1

        with open(save_file, 'w', encoding='utf-8') as output_file:
            output_file.write(text)
            # 在结尾添加文件名
            if end:
                output_file.write('\n\n\n')
                output_file.write(file_name)

    print('转换html：%s 为 markdown：%s' % (html_file, save_file))
    # print(save_file)
    return save_file


def doc():
    """
    打印模块说明文档
    """
    doc_text = """
    """
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=html_to_md)

    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------
    from time import time, sleep

    start_time = t1 = time()

    doc()

    print('运行时间 %.2f 秒' % (time() - start_time))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：excel_xlrd.py
#   版本：0.2
#   作者：lds
#   日期：2018-12-28
#   语言：Python 3.X
#   说明：读取excel文件内容
# ---------------------------------------

from time import time
from warnings import warn

# ReadExcel
import xlrd

(
    XL_CELL_EMPTY,
    XL_CELL_TEXT,
    XL_CELL_NUMBER,
    XL_CELL_DATE,
    XL_CELL_BOOLEAN,
    XL_CELL_ERROR,
    XL_CELL_BLANK,
) = range(7)

GEQU_TITLE_LIST = ['歌曲名', '歌曲', '歌名', '歌曲名称', '制品名称']
GESHOU_TITLE_LIST = ['歌手名', '歌手', '演唱', '表演者', '艺人名称', '演唱者', '表演者名称', '艺人']


class ReadXlsx(object):

    def __init__(self, *args, **kwargs):
        """
        读取 Excel 文件的类

        xls_file = r"(数据库标准格式20180424）2.xlsx"

        xls = ReadXlsx(xls_file)
        # time_fmt = "%Y-%m"
        print(xls.sheet_names)
        print(xls.title)
        print(xls.max_row)
        print(xls.max_column)

        # print(xls.datasets)
        # for i in xls.datasets:
        #     print(i)

        for i in xls.values():
            print(i)

        # print(xls.set_sheet(10)) # 这个要判断下是否成功
        print('next_sheet ---------------------------------')
        print(xls.next_sheet(),xls.index)
        print(xls.title)


        """
        # print('args', args,'kwargs',kwargs)

        # 打开的表索引
        try:
            self.index = kwargs.pop('index')
        except:
            self.index = 0

        # 数字转字符串

        # 日期格式
        self.time_fmt = "%Y-%m-%d"
        # self.time_fmt = "%Y-%m"

        self.excel = 'xls'
        self.wb = xlrd.open_workbook(*args, **kwargs)
        self.sheet_names = self.wb.sheet_names()
        self.set_sheet(self.index)
        # print(self.excel)

    @property
    def title(self):
        """  标题 """
        return self.sheet_names  # [self.index]

    @property
    def max_row(self):
        """  行 """
        return self.sheet.nrows

    @property
    def max_column(self):
        """  列 """
        return self.sheet.ncols

    @property
    def datasets(self):
        """
        sheet 的数据集（列表）
        xls 的表达式不能显示
        """
        return list(self.values())

    def values(self):
        """
        sheet 的数据集（列表）

        xlsx 空表格是 None
        xls  空表格是 ''

        xlsx 0 是 0
        xls  0 是 0.0

        xls 的表达式不能显示

        """
        # 遍历 sheet 中所有行row
        for row_index in range(self.sheet.nrows):
            # row_vals = self.sheet.row_values(row_index)
            # print('row %s is %s' % (curr_row, row_vals))
            row_vals = []
            for col_index in range(self.sheet.ncols):
                cell = self.sheet.cell(row_index, col_index)
                # print(xlrd.cellname(row_index, col_index), '-', end=' ')
                # print(cell.value, '-', end=' ')
                # print(cell.ctype)
                if XL_CELL_DATE == cell.ctype:
                    # 把 excel 时间格式转化为 字符串
                    cell_value = xlrd.xldate.xldate_as_datetime(cell.value, 0).strftime(self.time_fmt)
                    # print('XL_CELL_DATE',cell_value)
                elif XL_CELL_NUMBER == cell.ctype:
                    # 把 excel 数字转换为 字符串
                    if cell.value == int(cell.value):  # 检查是不是整数:
                        cell_value = str(int(cell.value))
                    else:
                        cell_value = str(cell.value)
                    # print('XL_CELL_NUMBER', cell.value,cell_value)
                elif XL_CELL_EMPTY == cell.ctype:
                    # 把 excel 空字符转换为 字符串 None
                    cell_value = ''
                    # print('XL_CELL_EMPTY', cell.value)
                else:
                    cell_value = cell.value
                row_vals.append(cell_value)
            yield row_vals

    def next_sheet(self):
        """ 下一个表 """
        return self.set_sheet(self.index + 1)

    def set_sheet(self, index=None):
        """ 设置表 """

        if index is not None:
            # 这里判断是否超过范围 调试了很久 ， 因为索引是从0开始的 啊 啊 啊 啊 啊
            if len(self.sheet_names) >= index + 1:
                self.index = index
            else:
                WarningInfo = "访问的表 %s 不存在" % index
                warn(WarningInfo)
                # if self.index == index: # 初始化设置的不存在的表
                #     self.index = 0
                return None

        self.sheet = self.wb.sheets()[self.index]

        return self.index


def xlrd_string(xlrdfloat):
    """转换 excel 的浮点数到字符串"""
    if xlrdfloat == int(xlrdfloat):  # 检查是不是整数:
        s = str(int(xlrdfloat))
    else:
        s = str(xlrdfloat)
    # s = str(cell.value)
    # s = float(cell.value)
    return s


def gequ_re(gequ):
    """处理歌曲名称
    最后修改时间：20181015
    """
    try:
        gequ = gequ.replace('	', ' ').replace('(', '（').replace(')', '）').replace('\n', '') \
            .replace('|', '/').replace(' ', ' ').replace(' （', '（') \
            .replace('  ', ' ').replace('  ', ' ').strip()
        # .replace('/', ' ').replace('+', ' ')
    except Exception as e:
        if isinstance(gequ, float):
            gequ = xlrd_string(gequ)
        else:
            print('问题：------------------- 歌名：%s -------------------' % (gequ))
    return gequ


def geshou_re(geshou):
    """处理表演者
    最后修改时间：20181015
    """
    geshou = geshou.replace('+', ' ').replace('/', ' ').replace('	', ' ').replace('(', '（').replace(
        ')', '）').replace('&', ' ').replace('|', ' ').replace('•', '·') \
        .replace('；', ' ').replace('\n', '').replace(',', ' ').replace(' ', ' ').replace(' （', '（') \
        .replace('  ', ' ').replace('  ', ' ').strip()
    if "（" in geshou:
        return geshou
    else:
        return geshou.replace('、', ' ').replace('  ', ' ')


def get_gequ_geshou_line(gequ_list, one_gequ=False):
    """在列表里面获取（歌曲名称）和（表演者）所在的行和列"""

    id = 0  # 记录列数
    line = 0
    gequ_id = 0
    geshou_id = 0

    for row_value in gequ_list:
        for i in row_value:
            if i:  # 内容为空就直接跳过
                i = str(i).strip()
                # print (i)
                # print (id)
                for gequ_ in range(0, len(GEQU_TITLE_LIST)):  # 获取歌曲名
                    # print (gequ[i])
                    if i == GEQU_TITLE_LIST[gequ_]:
                        # print (i, id, row_value)
                        gequ_id = id
                        # 是否只需要获取歌曲的行
                        if one_gequ:
                            return line, gequ_id
                        else:
                            id = 0  # 记录列数
                            for i in row_value:
                                if i:  # 内容为空就直接跳过
                                    i = str(i).strip()
                                    for geshou_ in range(0, len(GESHOU_TITLE_LIST)):  # 找到歌曲的时候才去获取歌手名
                                        # print (geshou[i])
                                        if i == GESHOU_TITLE_LIST[geshou_]:
                                            # print (i, id)
                                            geshou_id = id
                                            return line, gequ_id, geshou_id
                                id += 1
            id += 1
        id = 0
        line += 1
    if one_gequ:
        return None, gequ_id
    else:
        return None, gequ_id, geshou_id


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=ReadXlsx)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=xlrd_string)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=gequ_re)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=geshou_re)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_gequ_geshou_line)

    print(doc_text)


if __name__ == "__main__":
    t1 = time()
    xls_file = r"(数据库标准格式20180424）2.xlsx"

    xls = ReadXlsx(xls_file)
    # time_fmt = "%Y-%m"
    print(xls.sheet_names)
    print(xls.title)
    print(xls.max_row)
    print(xls.max_column)

    # print(xls.datasets)
    # for i in xls.datasets:
    #     print(i)

    for i in xls.values():
        print(i)

    # print(xls.set_sheet(10)) # 这个要判断下是否成功
    print('next_sheet ---------------------------------')
    print(xls.next_sheet(), xls.index)
    print(xls.title)

    print('用时 %.2f 秒' % (time() - t1))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：util.py
#   版本：0.3
#   作者：lds
#   日期：2019-01-21
#   语言：Python 3.X
#   说明：常用的函数集合
# ---------------------------------------
import os
import sys
import random

# 最后修改时间：20181016
CLEAN_STR = "	", " ", "(", ")", "（", "）", " ", "|", "/", "+", "&", "•", "；", " ", "＆", "　", "<", ">" \
    , "、", "\n", "\"", "?", "？", "*", ",", "《", "》", "-"


# https://stackoverflow.com/questions/8047204/django-script-to-access-model-objects-without-using-manage-py-shell
def django_setup(project_name=None, site_path=None):
    """
    设置 Django 运行环境

    from ilds.django.util import django_setup
    django_setup(r'mysite', site_path=None)
    """

    if site_path is not None:
        sys.path.insert(0, site_path)

    if project_name is None:
        project_name = os.path.split(os.path.dirname(__file__))[-1]
    print('项目：', project_name)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "{}.settings".format(project_name))
    try:
        import django
        django.setup()
    except ModuleNotFoundError as e:
        print("注：如果找不到 Django，请安装它: pip install django\n错误提示：", e)
        exit()


def random_key():
    """
    生成 Django 使用的 SECRET_KEY

    from ilds.django.util import random_key
    print(random_key())
    """

    return ''.join(
        [random.SystemRandom().choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += 'CLEAN_STR\n    需要清洗的字符（用在歌曲名，表演者匹配的时候）\n\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=django_setup)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=random_key)
    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------
    from time import time, sleep

    start_time = t1 = time()

    doc()

    print('运行时间 %.2f 秒' % (time() - start_time))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：json.py
#   版本：0.1
#   作者：lds
#   日期：2018-7-4
#   语言：Python 3.X
#   说明：处理 json 的函数集合
# ---------------------------------------

import os
import json


def json_read(path):
    """
    读取 json 文件
    """
    try:
        if not os.path.exists(path):
            return False
        with open(path, 'r', encoding='utf-8') as f:
            obj = json.load(f)
        return obj
    except Exception as e:
        print('json_read',e)
        return False


def json_save(obj, path):
    """
    保存 json 文件
    """

    if obj is None:
        return False
    else:
        try:
            with open(path, 'w', encoding='utf-8') as f:
                json.dump(obj, f, ensure_ascii=False, indent=2)
            return True
        except Exception as e:
            print('json_save', e)
            return False


def doc():
    """
    打印模块说明文档
    """
    doc_text = """
    """
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=json_read)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=json_save)

    print(doc_text)


if __name__ == '__main__':

    # 记录运行时间 --------------------------------------------------
    from time import time, sleep
    start_time = t1 = time()

    doc()

    print('运行时间 %.2f 秒' % (time() - start_time))



from collections import OrderedDict


# from django.apps import apps
# from django.db.models.base import ModelBase
from django.conf import settings

from django.forms.models import model_to_dict  # 获取模型实例的字典
# from django.db.models.base import ModelBase
from django.conf import settings
# # from django.db.models import FileField
from django.db.models import Sum

# from django.apps import apps
# group_by 这个分组 我喜欢
from django.db.models import Count
from django.db.models import QuerySet
# # from django.db.models import FileField
from django.db.models import Sum


import operator
from functools import reduce
from django.db import models

"""
20181203 添加 搜索多个字段的函数
20181203 ModelFields 添加 获取模型对象
20181203 添加了获取模型的字段，名字和类型的类
20181130 添加 确认是否修改线上数据库（本地操作，因为如果没有修改会直接退出，防止误操作）
20170814 整理了下模型调用
20170814 把模型操作分离出来，以后调用模型都通过这里
20180807 去掉不通用的模型操作，并改成共享库

批量导入例子
loadList = []
kwargs = 需要添加的字段字典
loadList.append(需要添加的库模型(**kwargs))
print('成功导入 %s 行' % len(models_ku.objects.bulk_create(loadList)))
"""


def get_search_results(queryset, search_fields, search_term):
    """
    搜索，返回包含查询集的
    来自：from django.contrib.admin.options import ModelAdmin
    """

    # 应用关键字搜索。
    def construct_search(field_name):
        if field_name.startswith('^'):
            return "%s__istartswith" % field_name[1:]
        elif field_name.startswith('='):
            return "%s__iexact" % field_name[1:]
        elif field_name.startswith('@'):
            return "%s__search" % field_name[1:]
        else:
            return "%s__icontains" % field_name

    if search_fields and search_term:
        orm_lookups = [construct_search(str(search_field))
                       for search_field in search_fields]
        for bit in search_term.split():
            or_queries = [models.Q(**{orm_lookup: bit})
                          for orm_lookup in orm_lookups]
            #
            queryset = queryset.filter(reduce(operator.or_, or_queries))

    return queryset


class ModelFields():
    """
    获取模型的字段，名字和类型。
    可以用来替代：
        get_model_verbose_name_dict
        get_model_name_dict
        get_model_verbose_names
        get_model_verbose_namesget_model_fields
        get_model_fields

    例子：
    from ilds.django.model import ModelFields
    print(ModelFields(mymodel).verbose_to_type('标题'))
    for i in ModelFields(mymodel).iter('type'):
    print(i)
    """

    def __init__(self, model, exclude=None):
        """
        初始化
        """
        self.field_list = []
        self.verbose_list = []
        self.type_list = []
        self.fields = []

        if exclude is None:
            exclude = []
        for f in model._meta.fields:
            field_name = f.name
            verbose_name = f.verbose_name
            type_name = type(f).__name__
            if field_name in exclude or verbose_name in exclude or type_name in exclude:
                continue
            self.field_list.append(field_name)
            self.verbose_list.append(verbose_name)
            self.type_list.append(type_name)
            self.fields.append(f)

        self.count = len(self.field_list)
        # print(self.field_list)
        # print(self.verbose_list)
        # print(self.type_list)

    def field_to_verbose(self, field):
        """通过 field_name 获取 verbose_name"""
        if field in self.field_list:
            return self.verbose_list[self.field_list.index(field)]

    def field_to_type(self, field):
        """通过 field_name 获取 type_name"""
        if field in self.field_list:
            return self.type_list[self.field_list.index(field)]

    def verbose_to_field(self, field):
        """通过 verbose_name 获取 field_name"""
        if field in self.verbose_list:
            return self.field_list[self.verbose_list.index(field)]

    def verbose_to_type(self, field):
        """通过 verbose_name 获取 type_name"""
        if field in self.verbose_list:
            return self.type_list[self.verbose_list.index(field)]

    def get_field(self, name):
        """通过 名字获取 field 属性
        按 field_name -> verbose_name 的顺序查找，
        已经找到就不在继续查找，没有找到返回空
        """
        if name in self.field_list:
            return self.fields[self.field_list.index(name)]
        elif name in self.verbose_list:
            return self.fields[self.verbose_list.index(name)]

    def iter(self, *args):
        """获取指定字段的生成器"""
        iter_dict = {
            'field': self.field_list,
            'verbose': self.verbose_list,
            'type': self.type_list,
        }
        if not args:
            args = ['field']
        else:
            if [arg for arg in args if arg not in iter_dict]:
                raise ValueError(f"参数错误：{args}，可用参数为：'field', 'verbose', 'type'")

        # print([iter_dict[arg][1] for arg in args])
        # print([i for i in range(self.count)])
        return ([iter_dict[arg][i] for arg in args] for i in range(self.count))


# 从上面改成输入 模型 和 排除
def get_model_verbose_name_dict(modelobj, exclude=None):
    """
    获取 model 的 verbose_name 和 name 的字段

    :param modelobj: 模型
    :param exclude: 排除
    :return: 键为 verbose_name，值为 name 的字典
    """

    filed = modelobj._meta.fields
    # print(filed)

    if exclude is None:
        params = [f for f in filed]
    else:
        params = [f for f in filed if f.name not in exclude]

    field_dict = OrderedDict()
    for i in params:
        field_dict[i.verbose_name] = i.name

    return field_dict


def get_model_name_dict(modelobj, exclude=None):
    """
    获取 model 的 name 字段 和 verbose_name

    :param modelobj: 模型
    :param exclude: 排除
    :return: 键为 name ，值为 verbose_name 的字典
    """

    filed = modelobj._meta.fields
    # print(filed)

    if exclude is None:
        params = [f for f in filed]
    else:
        params = [f for f in filed if f.name not in exclude]

    field_dict = OrderedDict()
    for i in params:
        field_dict[i.name] = i.verbose_name

    return field_dict


def get_model_verbose_names(modelobj, exclude=None):
    """
    获取数据模型的 verbose_name 名字列表

    :param modelobj: 模型
    :param exclude: 排除
    :return: 返回 verbose_name 列表
    """
    if exclude is None:
        exclude = []
    return [f.verbose_name for f in modelobj._meta.fields if f.name not in exclude]


def get_model_fields(modelobj, exclude=None):
    """
    获取数据模型的字段名列表

    :param modelobj: 模型
    :param exclude: 排除
    :return: 返回字段名字列表
    """
    if exclude is None:
        exclude = []
    return [f.name for f in modelobj._meta.fields if f.name not in exclude]


def get_fieldfile_path(_fieldfile):
    """
    获取 模型 FileField 路径
    """
    # print(_fieldfile.path)
    # print(_fieldfile.upfile.name)
    # print(_fieldfile.read())
    try:
        return _fieldfile.path
    except:
        return None


# def model_to_dict2(model_queryset, exclude):
#     model_dict = model_to_dict(model_queryset, exclude=exclude)

def get_queryset_sum(queryset, field_list, *args):
    """
    计算 QuerySet 的和 参数 QuerySet 需要求和的字段  需要显示的字段
    """

    kwargs = {}

    try:
        # 获取需要求和的内容
        for i in field_list:
            # print(i)
            if '_quanli_' in i:
                # print('_quanli_')
                continue
            elif i.endswith('_int') or i.endswith('_float'):
                if 'rmb_int' in i:
                    pass
                elif 'fencheng_int' in i:
                    pass
                else:
                    # print(i)
                    kwargs[i] = Sum(i)
        if kwargs:
            return queryset.values(*args).annotate(**kwargs)
        else:
            return None
    except:
        pass
    return None


def group_by(query_set, group_by):
    """
    util:django 获取分类列表 ( 对某些取到的QuerySet分组)

    例如获取厂牌名称
    print(group_by(objs,'shouquan_changpai'))
    》 ['厂牌名称1', '厂牌名称2', ...]

    :param query_set:
    :param group_by: 需要分组的字段
    :return: 分组字段的内容列表
    """

    assert isinstance(query_set, QuerySet)
    django_groups = query_set.values(group_by).annotate(Count(group_by))
    groups = []
    for dict_ in django_groups:
        # print(dict_)
        groups.append(dict_.get(group_by))
    return groups


def get_queryset_date_range(queryset, date_field='shujuriqi_date'):
    """
    获取 QuerySet 的日期间隔 参数 queryset，日期的字段
    """

    fromdate = queryset.order_by(date_field)[:1].values_list(date_field)[0][0].strftime("%Y-%m-%d")
    todate = queryset.order_by('-' + date_field)[:1].values_list(date_field)[0][0].strftime("%Y-%m-%d")
    return fromdate, todate


def confirm_db():
    """
    确认是否修改线上数据库
    本地操作，因为如果没有修改会直接退出，防止误操作
    """
    import wx
    app = wx.App()
    if not settings.DATABASES['default']['ENGINE'] == 'django.db.backends.sqlite3':
        dlg = wx.MessageDialog(None, "正在修改服务器数据库，是否继续？", "警告：不是连接的本地数据库", wx.YES_NO | wx.ICON_QUESTION)
        if dlg.ShowModal() == wx.ID_YES:
            print("修改服务器数据库......")
        else:
            exit()


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=ModelFields)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_model_verbose_name_dict)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_model_name_dict)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_model_verbose_names)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_model_fields)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_fieldfile_path)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_queryset_sum)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=group_by)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_queryset_date_range)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=confirm_db)

    print(doc_text)


if __name__ == "__main__":
    from time import time

    t1 = time()

    doc()

    print('用时 %.2f 秒' % (time() - t1))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：time.py
#   版本：0.5
#   作者：lds
#   日期：2018-11-28
#   语言：Python 3.X
#   说明：处理时间截的函数集合
# ---------------------------------------
import re
import time
import datetime


def timestamp_to_date(time_stamp, format_string="%Y-%m-%d %H:%M:%S", tz=None):
    """
    将 10 位时间戳转换为时间字符串，默认为 2018-01-23 01:23:45 格式
    """
    # time_array = time.localtime(time_stamp)
    # str_date = time.strftime(format_string, time_array)
    # 时间戳转换成字符串日期时间(另一种方法)
    d = datetime.datetime.fromtimestamp(time_stamp, tz)
    str_date = d.strftime(format_string)
    return str_date


def timestamp13_to_date(time_stamp, format_string="%Y-%m-%d %H:%M:%S", tz=None):
    """
    将 13 位时间戳转换为时间字符串，默认为 2018-01-23 01:23:45 格式
    """
    if len(str(time_stamp)) != 13:
        raise ValueError(time_stamp + '不是有效的 13 位 unix 时间戳')
    time_stamp = float(time_stamp) / 1000
    return timestamp_to_date(time_stamp, format_string, tz)


def date_to_timestamp(date, format_string="%Y-%m-%d %H:%M:%S"):
    """
    将时间字符串转换为 10 位时间戳，时间字符串默认为 2018-01-23 01:23:45 格式
    """
    time_array = time.strptime(date, format_string)
    time_stamp = int(time.mktime(time_array))
    return time_stamp


def now_to_timestamp(digits=10):
    """
    生成当前时间的时间戳，只有一个参数即时间戳的位数，默认为 10 位，
    输入位数即生成相应位数的时间戳，比如可以生成常用的13位时间戳
    """
    time_stamp = time.time()
    digits = 10 ** (digits - 10)
    time_stamp = int(round(time_stamp * digits))
    return time_stamp


def timestamp_to_timestamp10(time_stamp):
    """
    将时间戳规范为10位时间戳
    """
    time_stamp = int(time_stamp * (10 ** (10 - len(str(time_stamp)))))
    return time_stamp


def now_to_date(format_string="%Y-%m-%d %H:%M:%S"):
    """
    将当前时间转换为时间字符串，默认为 2018-01-23 01:23:45 格式
    """
    time_stamp = int(time.time())
    time_array = time.localtime(time_stamp)
    str_date = time.strftime(format_string, time_array)
    return str_date


def get_now():
    """
    获取当前日期和时间
    :return: 格式 2018-11-28 15:03:08
    """
    return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")


def get_now_date():
    """
    获取当前日期
    :return: 格式 2018-11-28
    """
    return datetime.datetime.now().strftime("%Y-%m-%d")


def get_now_time():
    """
    获取当前时间
    :return: 格式 15:03:08
    """
    return datetime.datetime.now().strftime("%H:%M:%S")


def date_style_transfomation(date, format_string1="%Y-%m-%d %H:%M:%S", format_string2="%Y-%m-%d %H-%M-%S"):
    """
    不同时间格式字符串的转换
    """
    time_array = time.strptime(date, format_string1)
    str_date = time.strftime(format_string2, time_array)
    return str_date


def is_vaild_date(date):
    """
    判断是否是一个有效的日期字符串
    """
    try:
        if ":" in date:
            time.strptime(date, "%Y-%m-%d %H:%M:%S")
        else:
            time.strptime(date, "%Y-%m-%d")
        return True
    except:
        return False


def form_time_to_year_mon_day(_time):
    """
    解析数字形式的日期 为 year-mon-day
    """
    time_str = str(_time)
    year_mon_day = None
    if len(time_str) == 6:
        year_s = time_str[:4]
        mon_s = time_str[4:6]
        # print((int(year_s), int(mon_s), int(day_s)))
        year_mon_day = '%s-%s-01' % (year_s, mon_s)
    elif len(time_str) == 8:
        year_s = time_str[:4]
        mon_s = time_str[4:6]
        day_s = time_str[6:8]
        # print((int(year_s), int(mon_s), int(day_s)))
        year_mon_day = '%s-%s-%s' % (year_s, mon_s, day_s)

    if is_vaild_date(year_mon_day):
        return year_mon_day
    else:
        raise ValueError(time_str + '不是有效的日期')


def date_from_str(date_str):
    """
    从字符串返回日期时间对象，格式为 YYYYMMDD 或（现在|今日）+ - ] [0-9]（天|周|月|年）？（S）
    Return a datetime object from a string in the format YYYYMMDD or
    (now|today)[+-][0-9](day|week|month|year)(s)?
    """
    today = datetime.date.today()
    if date_str in ('now', 'today'):
        return today
    if date_str == 'yesterday':
        return today - datetime.timedelta(days=1)
    match = re.match(r'(now|today)(?P<sign>[+-])(?P<time>\d+)(?P<unit>day|week|month|year)(s)?', date_str)
    if match is not None:
        sign = match.group('sign')
        time = int(match.group('time'))
        if sign == '-':
            time = -time
        unit = match.group('unit')
        # A bad approximation?
        if unit == 'month':
            unit = 'day'
            time *= 30
        elif unit == 'year':
            unit = 'day'
            time *= 365
        unit += 's'
        delta = datetime.timedelta(**{unit: time})
        return today + delta
    return datetime.datetime.strptime(date_str, '%Y%m%d').date()


def hyphenate_date(date_str):
    """
    将“YYYYMMDD”格式的日期转换为“YYYY-MM-DD”格式
    Convert a date in 'YYYYMMDD' format to 'YYYY-MM-DD' format
    """
    match = re.match(r'^(\d\d\d\d)(\d\d)(\d\d)$', date_str)
    if match is not None:
        return '-'.join(match.groups())
    else:
        return date_str


class DateRange(object):
    """
    表示两个日期之间的时间间隔
    Represents a time interval between two dates
    """

    def __init__(self, start=None, end=None):
        """start and end must be strings in the format accepted by date"""
        if start is not None:
            self.start = date_from_str(start)
        else:
            self.start = datetime.datetime.min.date()
        if end is not None:
            self.end = date_from_str(end)
        else:
            self.end = datetime.datetime.max.date()
        if self.start > self.end:
            raise ValueError('Date range: "%s" , the start date must be before the end date' % self)

    @classmethod
    def day(cls, day):
        """Returns a range that only contains the given day"""
        return cls(day, day)

    def __contains__(self, date):
        """Check if the date is in the range"""
        if not isinstance(date, datetime.date):
            date = date_from_str(date)
        return self.start <= date <= self.end

    def __str__(self):
        return '%s - %s' % (self.start.isoformat(), self.end.isoformat())


def srt_subtitles_timecode(seconds):
    """
    秒转换为时间字符串 01:02:03,000
    """
    return '%02d:%02d:%02d,%03d' % (seconds / 3600, (seconds % 3600) / 60, seconds % 60, (seconds % 1) * 1000)


def second_to_time_str(seconds):
    """
    秒转换为人类阅读的时间显示，用来显示已用时间
    例如：'1小时1分1.099秒'
    """
    time_str = ''
    hour = '%01d小时' % (seconds / 3600)
    minute = '%01d分' % ((seconds % 3600) / 60)

    if hour != '0小时':
        time_str += hour

    if minute != '0分':
        time_str += minute

    # seconds
    time_str += '%01d.%03d秒' % (seconds % 60, (seconds % 1) * 1000)

    return time_str


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=timestamp_to_date)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=timestamp13_to_date)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=date_to_timestamp)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=now_to_timestamp)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=timestamp_to_timestamp10)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=now_to_date)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_now)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_now_date)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_now_time)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=date_style_transfomation)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=is_vaild_date)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=form_time_to_year_mon_day)
    # youtube_dl\utils.py
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=date_from_str)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=hyphenate_date)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=DateRange)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=srt_subtitles_timecode)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=second_to_time_str)
    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------

    start_time = t1 = time.time()

    doc()

    # # 例子
    # print(now_to_date())
    # print(timestamp_to_date(1516641825))
    # print(timestamp13_to_date(1516641825000))
    # print(date_to_timestamp('2018-01-23 01:23:45'))
    # print(timestamp_to_timestamp10(1516641825000))
    # print(date_style_transfomation('2018-01-23 01:23:45'))
    # # 结果为：
    # # 2018-11-15 11:19:23
    # # 2018-01-23 01:23:45
    # # 2018-01-23 01:23:45
    # # 1516641825
    # # 1516641825
    # # 2018-01-23 01-23-45

    print('运行时间 %.2f 秒' % (time.time() - start_time))

"""


Python处理时间和时间戳的内置模块就有time，和datetime两个，本文先说time模块。

关于时间戳的几个概念

时间戳，根据1970年1月1日00:00:00开始按秒计算的偏移量。
时间元组（struct_time），包含9个元素。 

time.struct_time(tm_year=2017, tm_mon=10, tm_mday=1, tm_hour=14, tm_min=21, tm_sec=57, tm_wday=6, tm_yday=274, tm_isdst=0) 

时间格式字符串，字符串形式的时间。
time模块与时间戳和时间相关的重要函数

time.time() 生成当前的时间戳，格式为10位整数的浮点数。
time.strftime()根据时间元组生成时间格式化字符串。
time.strptime()根据时间格式化字符串生成时间元组。time.strptime()与time.strftime()为互操作。
time.localtime()根据时间戳生成当前时区的时间元组。
time.mktime()根据时间元组生成时间戳。


"""



# -*- coding: utf-8 -*-
__doc__ = 'Load browser cookies into a cookiejar'

# 获取浏览器cookie
# 主页：https://bitbucket.org/richardpenman/browsercookie
# 最近更新时间：2018-06-08

# 我最后修改日期：2018-11-26

# 需要的依赖
# pip install pycryptodome
# pip install keyring
# pip install lz4
# Windows 访问 FireFox 数据可能出错 更新sqlite
# pip install pysqlite

import os
import sys
import time
import glob

try:
    import cookielib
except ImportError:
    import http.cookiejar as cookielib
from contextlib import contextmanager
import tempfile

try:
    import json
except ImportError:
    import simplejson as json
try:
    import ConfigParser as configparser
except ImportError:
    import configparser

try:
    # should use pysqlite2 to read the cookies.sqlite on Windows
    # otherwise will raise the "sqlite3.DatabaseError: file is encrypted or is not a database" exception
    from pysqlite2 import dbapi2 as sqlite3
except ImportError:
    import sqlite3

import lz4.block
import keyring
from Crypto.Protocol.KDF import PBKDF2
from Crypto.Cipher import AES


class BrowserCookieError(Exception):
    pass


@contextmanager
def create_local_copy(cookie_file):
    """Make a local copy of the sqlite cookie database and return the new filename.
    This is necessary in case this database is still being written to while the user browses
    to avoid sqlite locking errors.
    """
    # check if cookie file exists
    if os.path.exists(cookie_file):
        # copy to random name in tmp folder
        tmp_cookie_file = tempfile.NamedTemporaryFile(suffix='.sqlite').name
        open(tmp_cookie_file, 'wb').write(open(cookie_file, 'rb').read())
        yield tmp_cookie_file
    else:
        raise BrowserCookieError('Can not find cookie file at: ' + cookie_file)

    os.remove(tmp_cookie_file)


class BrowserCookieLoader(object):
    def __init__(self, cookie_files=None):
        cookie_files = cookie_files or self.find_cookie_files()
        self.cookie_files = list(cookie_files)

    def find_cookie_files(self):
        """Return a list of cookie file locations valid for this loader"""
        raise NotImplementedError

    def get_cookies(self):
        """Return all cookies (May include duplicates from different sources)"""
        raise NotImplementedError

    def load(self):
        """Load cookies into a cookiejar"""
        # cookie_jar = cookielib.CookieJar()
        # LWPCookieJar 替换为 CookieJar 支持支持保存到文件，和从文件载入
        cookie_jar = cookielib.LWPCookieJar()
        for cookie in self.get_cookies():
            cookie_jar.set_cookie(cookie)
        return cookie_jar


class Chrome(BrowserCookieLoader):
    def __str__(self):
        return 'chrome'

    def find_cookie_files(self):
        for pattern in [
            os.path.expanduser('~/Library/Application Support/Google/Chrome/Default/Cookies'),
            os.path.expanduser('~/Library/Application Support/Vivaldi/Default/Cookies'),
            os.path.expanduser('~/.config/chromium/Default/Cookies'),
            os.path.expanduser('~/.config/chromium/Profile */Cookies'),
            os.path.expanduser('~/.config/google-chrome/Default/Cookies'),
            os.path.expanduser('~/.config/google-chrome/Profile */Cookies'),
            os.path.expanduser('~/.config/vivaldi/Default/Cookies'),
            os.path.join(os.getenv('APPDATA', ''), r'..\Local\Google\Chrome\User Data\Default\Cookies'),
            os.path.join(os.getenv('APPDATA', ''), r'..\Local\Vivaldi\User Data\Default\Cookies'),
        ]:
            for result in glob.glob(pattern):
                yield result

    def get_cookies(self):
        salt = b'saltysalt'
        length = 16
        if sys.platform == 'darwin':
            # running Chrome on OSX
            my_pass = keyring.get_password('Chrome Safe Storage', 'Chrome')
            my_pass = my_pass.encode('utf8')
            iterations = 1003
            key = PBKDF2(my_pass, salt, length, iterations)

        elif sys.platform.startswith('linux'):
            # running Chrome on Linux
            my_pass = 'peanuts'.encode('utf8')
            iterations = 1
            key = PBKDF2(my_pass, salt, length, iterations)

        elif sys.platform == 'win32':
            key = None
        else:
            raise BrowserCookieError('Unsupported operating system: ' + sys.platform)

        for cookie_file in self.cookie_files:
            with create_local_copy(cookie_file) as tmp_cookie_file:
                con = sqlite3.connect(tmp_cookie_file)
                cur = con.cursor()
                cur.execute('SELECT value FROM meta WHERE key = "version";')
                version = int(cur.fetchone()[0])
                query = 'SELECT host_key, path, is_secure, expires_utc, name, value, encrypted_value FROM cookies;'
                if version < 10:
                    query = query.replace('is_', '')
                cur.execute(query)
                for item in cur.fetchall():
                    host, path, secure, expires, name = item[:5]
                    value = self._decrypt(item[5], item[6], key=key)
                    yield create_cookie(host, path, secure, expires, name, value)
                con.close()

    def _decrypt(self, value, encrypted_value, key):
        """Decrypt encoded cookies
        """
        if (sys.platform == 'darwin') or sys.platform.startswith('linux'):
            if value or (encrypted_value[:3] != b'v10'):
                return value

            # Encrypted cookies should be prefixed with 'v10' according to the
            # Chromium code. Strip it off.
            encrypted_value = encrypted_value[3:]

            # Strip padding by taking off number indicated by padding
            # eg if last is '\x0e' then ord('\x0e') == 14, so take off 14.
            def clean(x):
                last = x[-1]
                if isinstance(last, int):
                    return x[:-last].decode('utf8')
                else:
                    return x[:-ord(last)].decode('utf8')

            iv = b' ' * 16
            cipher = AES.new(key, AES.MODE_CBC, IV=iv)
            decrypted = cipher.decrypt(encrypted_value)
            return clean(decrypted)
        else:
            # Must be win32 (on win32, all chrome cookies are encrypted)
            try:
                import win32crypt
            except ImportError:
                raise BrowserCookieError('win32crypt must be available to decrypt Chrome cookie on Windows')
            return win32crypt.CryptUnprotectData(encrypted_value, None, None, None, 0)[1].decode("utf-8")


class Firefox(BrowserCookieLoader):
    def __str__(self):
        return 'firefox'

    def parse_profile(self, profile):
        cp = configparser.ConfigParser()
        cp.read(profile)
        path = None
        for section in cp.sections():
            try:
                if cp.getboolean(section, 'IsRelative'):
                    path = os.path.dirname(profile) + '/' + cp.get(section, 'Path')
                else:
                    path = cp.get(section, 'Path')
                if cp.has_option(section, 'Default'):
                    return os.path.abspath(os.path.expanduser(path))
            except configparser.NoOptionError:
                pass
        if path:
            return os.path.abspath(os.path.expanduser(path))
        raise BrowserCookieError('No default Firefox profile found')

    def find_default_profile(self):
        if sys.platform == 'darwin':
            return glob.glob(os.path.expanduser('~/Library/Application Support/Firefox/profiles.ini'))
        elif sys.platform.startswith('linux'):
            return glob.glob(os.path.expanduser('~/.mozilla/firefox/profiles.ini'))
        elif sys.platform == 'win32':
            return glob.glob(os.path.join(os.getenv('APPDATA', ''), 'Mozilla/Firefox/profiles.ini'))
        else:
            raise BrowserCookieError('Unsupported operating system: ' + sys.platform)

    def find_cookie_files(self):
        profile = self.find_default_profile()
        if not profile:
            raise BrowserCookieError('Could not find default Firefox profile')
        path = self.parse_profile(profile[0])
        if not path:
            raise BrowserCookieError('Could not find path to default Firefox profile')
        cookie_files = glob.glob(os.path.expanduser(path + '/cookies.sqlite'))
        if cookie_files:
            return cookie_files
        else:
            raise BrowserCookieError('Failed to find Firefox cookies')

    def get_cookies(self):
        for cookie_file in self.cookie_files:
            with create_local_copy(cookie_file) as tmp_cookie_file:
                con = sqlite3.connect(tmp_cookie_file)
                cur = con.cursor()
                cur.execute('select host, path, isSecure, expiry, name, value from moz_cookies')

                for item in cur.fetchall():
                    yield create_cookie(*item)
                con.close()

                # current sessions are saved in sessionstore.js/recovery.json/recovery.jsonlz4
                session_files = (os.path.join(os.path.dirname(cookie_file), 'sessionstore.js'),
                                 os.path.join(os.path.dirname(cookie_file), 'sessionstore-backups', 'recovery.json'),
                                 os.path.join(os.path.dirname(cookie_file), 'sessionstore-backups', 'recovery.jsonlz4'))
                for file_path in session_files:
                    if os.path.exists(file_path):
                        if file_path.endswith('4'):
                            try:
                                session_file = open(file_path, 'rb')
                                # skip the first 8 bytes to avoid decompress failure (custom Mozilla header)
                                session_file.seek(8)
                                json_data = json.loads(lz4.block.decompress(session_file.read()).decode())
                            except IOError as e:
                                print('Could not read file:', str(e))
                            except ValueError as e:
                                print('Error parsing Firefox session file:', str(e))
                        else:
                            try:
                                json_data = json.loads(open(file_path, 'rb').read().decode('utf-8'))
                            except IOError as e:
                                print('Could not read file:', str(e))
                            except ValueError as e:
                                print('Error parsing firefox session JSON:', str(e))

                if 'json_data' in locals():
                    expires = str(int(time.time()) + 3600 * 24 * 7)
                    for window in json_data.get('windows', []):
                        for cookie in window.get('cookies', []):
                            yield create_cookie(cookie.get('host', ''), cookie.get('path', ''), False, expires,
                                                cookie.get('name', ''), cookie.get('value', ''))
                else:
                    print('Could not find any Firefox session files')


def create_cookie(host, path, secure, expires, name, value):
    """Shortcut function to create a cookie
    """
    return cookielib.Cookie(0, name, value, None, False, host, host.startswith('.'), host.startswith('.'), path, True,
                            secure, expires, False, None, None, {})


def chrome(cookie_file=None):
    """Returns a cookiejar of the cookies used by Chrome
    """
    return Chrome(cookie_file).load()


def firefox(cookie_file=None):
    """Returns a cookiejar of the cookies and sessions used by Firefox
    """
    return Firefox(cookie_file).load()


def _get_cookies():
    """Return all cookies from all browsers"""
    for klass in [Chrome, Firefox]:
        try:
            for cookie in klass().get_cookies():
                yield cookie
        except BrowserCookieError:
            pass


def load():
    """Try to load cookies from all supported browsers and return combined cookiejar
    """
    cookie_jar = cookielib.CookieJar()

    for cookie in sorted(_get_cookies(), key=lambda cookie: cookie.expires):
        cookie_jar.set_cookie(cookie)

    return cookie_jar


if __name__ == '__main__':
    import time

    import requests
    from http import cookiejar

    t1 = time.time()
    session = requests.session()
    # 初始化 session.cookies 为 LWPCookieJar 对象
    session.cookies = cookiejar.LWPCookieJar(filename='./cookies.txt')

    # session.get('https://www.baidu.com/s?wd=LWPCookieJar&ie=UTF-8')

    # 保存当前 Cookies
    # session.cookies.save()

    #  读取 Cookies 文件加载到 Session
    session.cookies.load(ignore_discard=True)

    cj = firefox()
    # cj = chrome()
    # cj = session.cookies  # 这个应该可以相互使用

    for cookie in cj:
        # print(dir(cookie))
        # if cookie.domain:
        #     print(type(cookie), cookie.domain)
        cookie_dict = {
            'path': cookie.path,
            'domain': cookie.domain,
            'name': cookie.name,
            'value': cookie.value,
            'expires': cookie.expires,
            # 下面是附加内容 还不清楚有什么用
            # 'version': cookie.version,
            # 'path_spec': cookie.path_spec,  # 无
            # 'domain_dot': cookie.domain_dot,  # 无
        }
        print(cookie_dict)
        # # 添加 cookie 到 selenium
        # try:
        #     driver.add_cookie(cookie_dict)
        # except Exception as e:
        #     print(e)  # , cookie_dict
    cj.save('001.txt')

    # print(cj.__dict__)

    print('用时 %.2f 秒' % (time.time() - t1))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：util.py
#   版本：0.2
#   作者：lds
#   日期：2018-11-07
#   语言：Python 3.X
#   说明：常用的函数集合
# ---------------------------------------


def print_doc(fun, is_all=True):
    """
    打印函数的文档信息
    """
    siyou = []
    siyou2 = []
    doc = []
    for name in dir(fun):
        if name.startswith('__'):
            if len(siyou)%6 == 0:
                siyou.append('\n')
            siyou.append(name)

        elif name.startswith('_'):
            if len(siyou2)%6 == 0:
                siyou2.append('\n')
            siyou2.append(name)
        else:
            if is_all:
                doc.append("%s " % name + "-"*78)
                doc.append(str(eval("fun.%s.__doc__" % name)))
                doc.append("="*78 + " %s\n" % name)
            else:
                doc.append(name)

    print(' '.join(siyou))
    print(' '.join(siyou2))
    print('\n'.join(doc))


def prints(frame, *args):
    """
    打印的时候，包括当前行信息，方便 pycharm 直接跳转到当前位置
    例子：
    prints(sys._getframe(), 1, dict)
    """
    print(f'File "{frame.f_code.co_filename}", line {frame.f_lineno}, {frame.f_code.co_name}\n   ', *args)


def dict_val_to_key(mydict):
    """
    字典键值互换
    """
    return dict([val,key] for key,val in mydict.items())


def list_to_dict(list1,list2):
    """
    把两个列表转换成相对应的字典
    """
    ku_field_row_dict = {}
    for i in range(len(list1)):
        # print(list1[i], list2[i])
        ku_field_row_dict[list1[i]] = list2[i]
    return ku_field_row_dict


def sort_dict(dict_, reverse=False):
    """
    按字典（键）重新排序
    :param dict_: 字典
    :param reverse: 排序方式 True False
    :return: 排序好的字典
    """
    dict_sort = {}

    for k in sorted(dict_.keys(), reverse=reverse):
        dict_sort[k] = dict_[k]

    return dict_sort


def get_kuohao_feijie(value):
    """
    输入带括号内容的内容 返回内容 和 括号内内容
    """
    str_kuohao = value.strip()
    if ("（") in str_kuohao:
        if "）" in str_kuohao:
            pos1 = str_kuohao.index("（")
            pos2 = str_kuohao.index("）")
            # print(pos1)
            # print(pos2)
            # print(len(str_kuohao))
            if (pos2 + 1) == len(str_kuohao):
                kuohao1 = str_kuohao[: pos1].strip()
                kuohao2 = str_kuohao[pos1 + 1: pos2].strip()
                return kuohao1, kuohao2
            else:
                print("歌曲括号后面有内容",value)
                return value
                #
        else:
            print("歌曲括号不匹配",value)
            return value


def xl_col_to_name(col_num, col_abs=False):
    """
    将零索引列单元格引用转换为字符串。

    来自 ： xlsxwriter 中的 utility.py

    Args:
       col:     列 数字
       col_abs: 用于使列绝对的可选标志。布尔。

    Returns:
        列样式字符串。

    """
    col_num += 1  # 改为 1-index.
    col_str = ''
    col_abs = '$' if col_abs else ''

    while col_num:
        # 提醒从 1 .. 26 设置序号
        remainder = col_num % 26

        if remainder == 0:
            remainder = 26

        # 将余数转换为字符。
        col_letter = chr(ord('A') + remainder - 1)

        # 从右到左累积列字母。
        col_str = col_letter + col_str

        # 获得下一个数量级
        col_num = int((col_num - 1) / 26)

    return col_abs + col_str


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=print_doc)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=prints)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=dict_val_to_key)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=list_to_dict)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=sort_dict)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_kuohao_feijie)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=xl_col_to_name)
    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------
    from time import time, sleep

    start_time = t1 = time()

    doc()

    print('运行时间 %.2f 秒' % (time() - start_time))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：user.py
#   版本：0.1
#   作者：lds
#   日期：2019-01-21
#   语言：Python 3.X
#   说明：django 用户相关的函数集合
# ---------------------------------------

from django.contrib.auth.models import User
from django.contrib.auth.hashers import make_password  # , check_password


def add_superuser(username, email, password):
    """
    添加 django 超级用户

    from ilds.django.user import add_superuser
    add_superuser(username='admin', email='admin@qq.com', password='123456')
    """
    if not password.startswith('pbkdf2_'):
        password = make_password(password)

    # 添加一个超级用户数据
    add_user = User.objects.create(**{
        'password': password,
        'last_login': None,
        'is_superuser': True,
        'username': username,
        'email': email,
        'is_staff': True,
        'is_active': True})

    try:
        from allauth.account.models import EmailAddress  # django-allauth
        EmailAddress.objects.create(**{
            'user': add_user,
            'email': email,
            'verified': True,
            'primary': True})
    except Exception as e:
        print(e)

    return True


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=add_superuser)
    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------
    from time import time, sleep

    start_time = t1 = time()

    doc()

    print('运行时间 %.2f 秒' % (time() - start_time))



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：excel_xlsx.py
#   版本：0.2
#   作者：lds
#   日期：2019-01-14
#   语言：Python 3.X
#   说明：读取excel文件内容
# ---------------------------------------

from time import time
from warnings import warn

# ReadExcel
from openpyxl import load_workbook
from openpyxl.styles import numbers, is_date_format

(
    XL_CELL_EMPTY,
    XL_CELL_TEXT,
    XL_CELL_NUMBER,
    XL_CELL_DATE,
    XL_CELL_BOOLEAN,
    XL_CELL_ERROR,
    XL_CELL_BLANK,
) = range(7)


def get_title_style(workbook, style_colour='#daeef3', is_bold=False):
    """
    获取自定义颜色的标题风格
    :param workbook:
    :param style_colour: 默认颜色为什么浅色
    :param is_bold: 是否设置粗体字体
    :return:
    """
    head_style = workbook.add_format()
    # 设置粗体
    if is_bold:
        head_style.set_bold()
    # 设置对齐
    head_style.set_align('center')
    head_style.set_align('vcenter')
    # 设置颜色
    head_style.set_bg_color(style_colour)
    # 设置边框
    head_style.set_border()
    # 调整行的高度以适应文本
    head_style.set_text_wrap()
    return head_style


class ReadXlsx(object):

    def __init__(self, *args, **kwargs):
        """
        读取 Excel 文件的类 用 openpyxl 重写

        # 例子
        xlsx_file = r"E:\my - 数据\其他\20180731 阿玲 参考\申请单G1.5(1)(1).xlsx"
        # data_only=True 的时候不读取公式 , index=10
        # 读取大文件的时候添加 read_only=True
        # xlsx = ReadExcel(xlsx_file, data_only=True)

        xlsx = ReadXlsx(xlsx_file)
        # time_fmt = "%Y-%m"
        xlsx.debug = True
        # print(xlsx.sheet_names)

        for i in range(len(xlsx.sheet_names)):
            xlsx.max_row
            xlsx.max_column
            print('next_sheet ---------------------------------')
            xlsx.next_sheet()

        # print(xlsx.datasets())
        # for i in xlsx.datasets():
        #     print(i)

        # for i in xlsx.values():
        #     print(i)

        print(xlsx.set_sheet(10)) # 这个要判断下是否成功
        print(xlsx.next_sheet(), xlsx.index)

        print(xlsx.title)

        """
        # print('args', args,'kwargs',kwargs)

        # 打开的表索引
        try:
            self.index = kwargs.pop('index')
        except:
            self.index = 0

        # 数字转字符串

        # 日期格式
        self.time_fmt = "%Y-%m-%d" # YYYY-MM-DD
            # self.time_fmt = "%Y-%m"

        # self.excel = 'xlsx'
        self.wb = load_workbook(*args,**kwargs)
        self.sheet_names = self.wb.sheetnames
        self.set_sheet(self.index)
        # print(self.excel)

        self.debug = False

    @property
    def title(self):
        """  标题 """
        return self.sheet_names[self.index]

    @property
    def max_row(self):
        """  行 """
        if self.debug: print(f"title {self.title} index {self.index} 行 {self.sheet.max_row}")
        return self.sheet.max_row

    @property
    def max_column(self):
        """  列 """
        if self.debug:print(f"title {self.title} index {self.index} 列 {self.sheet.max_column}")
        return self.sheet.max_column

    # @property
    def datasets(self):
        """
        sheet 的数据集（列表）
        xls 的表达式不能显示
        """
        return list(self.values())

    def values(self):
        """
        sheet 的数据集（列表）
        xlsx 空表格是 None
        xls  空表格是 ''
        xlsx 0 是 0
        xls  0 是 0.0
        xls 的表达式不能显示
        # 返回可迭代结果
        """
        # rows 是可迭代的生成器
        for row in self.sheet.rows:
            row_vals = []
            # row_vals = [c.value for c in row]
            # print(row)
            for c in row:
                # fill = c.fill
                # fg_color = fill.start_color.rgb  # start_color fgColor
                # if 'Values must be of type' in str(fg_color):
                #     fg_color = '00000000'
                # print(fg_color)
                # bg_color = fill.end_color.rgb  # end_color bgColor
                # print(bg_color)
                # print(c.data_type, c.number_format, is_date_format(c.number_format), c.value)
                if c.data_type == "n" :
                    if c.number_format != "General" and is_date_format(c.number_format):
                        # print('number_format 日期',c.number_format,c.value)
                        if c.value is not None:
                            cell_value = c.value.strftime(self.time_fmt)
                            # print(cell_value)
                        else:
                            cell_value = ''
                    else:
                        if c.value is not None:
                            cell_value = str(c.value)  # 数字转换为字符串
                        else:
                            cell_value = ''
                elif c.data_type == "d" :
                    cell_value = c.value.strftime(self.time_fmt)
                else:
                    cell_value = c.value
                row_vals.append(cell_value)
            # print(row_vals)
            yield row_vals

    def next_sheet(self):
        """ 下一个表 """
        return self.set_sheet(self.index + 1)

    def set_sheet(self, index=None):
        """ 设置表 """

        if index is not None:
            # 这里判断是否超过范围 调试了很久 ， 因为索引是从0开始的 啊 啊 啊 啊 啊
            if len(self.sheet_names) >= index + 1:
                self.index = index
            else:
                # WarningInfo = "访问的表 %s 不存在" % index
                # warn(WarningInfo)
                # if self.index == index: # 初始化设置的不存在的表
                #     self.index = 0
                return None

        # 通过名字获取 sheet
        self.sheet = self.wb[self.sheet_names[self.index]]

        return self.index


def doc():
    """
    打印模块说明文档
    """
    doc_text = """"""
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=ReadXlsx)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_title_style)

    print(doc_text)

if __name__ == "__main__":
    t1 = time()

    # 例子
    xlsx_file = r"E:\my - 数据\其他\20180731 阿玲 参考\申请单G1.5(1)(1).xlsx"
    # data_only=True 的时候不读取公式 , index=10
    # 读取大文件的时候添加 read_only=True
    # xlsx = ReadExcel(xlsx_file, data_only=True)

    xlsx = ReadXlsx(xlsx_file)
    # time_fmt = "%Y-%m"
    xlsx.debug = True
    # print(xlsx.sheet_names)

    for i in range(len(xlsx.sheet_names)):
        xlsx.max_row
        xlsx.max_column
        print('next_sheet ---------------------------------')
        xlsx.next_sheet()

    # print(xlsx.datasets())
    # for i in xlsx.datasets():
    #     print(i)

    # for i in xlsx.values():
    #     print(i)

    print(xlsx.set_sheet(10)) # 这个要判断下是否成功
    print(xlsx.next_sheet(), xlsx.index)

    print(xlsx.title)

    print('用时 %.2f 秒' % (time() - t1))





from setuptools import setup, find_packages

# import os, sys


version = '0.1.36'

# find_packages()
# 对于简单工程来说，手动增加packages参数很容易，这个函数默认在和setup.py同一目录下搜索各个含有 __init__.py的包。
# 其实我们可以将包统一放在一个src目录中，另外，这个包内可能还有 aaa.txt 文件和 data 数据文件夹。
# 另外，也可以排除一些特定的包：
# find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"])

# 检查 setup.py 是不是正确，如果只输出 running check，那么就ok了
# python setup.py check

# 打包
# python setup.py sdist
# 把安装包加入到官方的pip服务器
# 首先注册一个pip账号（https://pypi.python.org/pypi）
# 注册自己的包名以及上传需要运行命令 python setup.py register sdist upload 按照要求填写自己的用户名密码即可。
# 此时只需要 pip install mypackage 就能够安装自己的包，超级方便。
# 自己的包修改之后，修改 setup.py 中的版本号即可再次更新上传。
# 有时更新较频繁会导致 pip 从缓存中获取数据而不是服务器上的最新版本，运行 pip 时加上 --no-cache-dir 选项即可解决。


# setup.py文件的使用：
# python setup.py build # 编译
# python setup.py install    # 安装
# python setup.py sdist     # 制作分发包
# python setup.py sdist build  # 打包为tar.gz
# python setup.py sdist bdist_wheel # 打包 wheels 格式的包
# python setup.py bdist_wininst # 制作windows下的分发包
# python setup.py bdist_rpm

# https://pypi.org/project/ilds/
"""

pip install -U ilds
pip --no-cache-dir install -U ilds


echo 使用 twine 上传到官方的pip服务器:
echo 已经不需要 cd tools\ilds
rmdir /S/Q build
rmdir /S/Q dist
python setup.py sdist bdist_wheel
echo 上传到PyPI:
twine upload dist/*


# 本地安装（不推荐在生产环境使用）
PyCharm > Terminal
cd tools\lds&python setup.py sdist
pip install -U dist\lds-0.0.8.tar.gz



pip install -U "/Users/lds/Library/Mobile Documents/iCloud~com~omz-software~Pythonista3/Documents/tools/lds/dist/lds-0.0.8.tar.gz"

"""


# twine upload dist/* 使用 twine 上传

# error: invalid command 'bdist_wheel' 需要安装wheel ： pip install -U wheel
# 读取 README.md 文件内容
def read_md_convert(f):
    return convert(f, 'md')


def read_md_open(f):
    return open(f, 'r').read()


try:
    from pypandoc import convert

    read_md = read_md_convert
except ImportError:
    read_md = read_md_open

setup(
    # 名称
    name="ilds",
    # 版本
    version=version,
    # version=".".join(map(str, __import__('html2text').__version__)),
    # 关键字列表
    # keywords = ("test", "xxx"),
    # 简单描述
    description="常用模块的集合，为了多平台，多电脑调用方便!",
    # 详细描述
    # long_description = "常用爬虫功能的封装",
    # long_description = read_md('README.md'),
    long_description=open('README.rst', encoding='utf-8').read(),
    # 授权信息
    license="GNU GPL 3",

    # 官网地址
    url="https://github.com/ldsxp/ilds",
    # 程序的下载地址
    download_url="https://pypi.org/project/ilds",
    # 作者
    author="lds",
    # 作者的邮箱地址
    author_email="85176878@qq.com",
    # 维护者
    # maintainer = "lds2",
    # 维护者的邮箱地址
    # maintainer_email = "85176878@qq.com",

    # 需要处理的包目录（包含__init__.py的文件夹）
    packages=find_packages(),
    # packages = ['lds'],
    # 需要打包的python文件列表
    # py_modules = "any",
    # 需要打包的数据文件，如图片，配置文件等
    # data_files = "a.jpg"
    # 使用 MANIFEST.in 设置 include_package_data
    # include_package_data = True,
    # 软件平台列表
    platforms="any",
    # 所属分类列表
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    # 需要安装的依赖包
    install_requires=[
        'xlrd>=1.1.0',
        'colorama',
        'chardet',
        'pyperclip',
        # 'setuptools>=16.0',
    ],

    # 此项需要，否则卸载时报windows error
    # zip_safe = False

    #  安装时需要执行的脚步列表
    # scripts = [],
    # 告诉setuptools哪些目录下的文件被映射到哪个源码包。
    # package_dir = {'': 'lib'} # 表示“root package”中的模块都在lib 目录中
    # 定义依赖哪些模块
    # requires
    # 定义可以为哪些模块提供依赖
    # provides
    #  动态发现服务和插件
    # entry_points = {
    #     'console_scripts': [
    #         'test = test.help:main'
    #     ]
    # }
    # entry_points="""
    #     [console_scripts]
    #     html2text=html2text.cli:main
    # """,
)



# -*- coding: utf-8 -*-
#
# ---------------------------------------
#   程序：file.py
#   版本：0.4
#   作者：lds
#   日期：2019-01-18
#   语言：Python 3.X
#   说明：常用的文件操作函数集合
# ---------------------------------------


import os
import sys
import random
import shutil
# get_file_md5
import hashlib
# get_encoding
import chardet
# validateTitle
import re
from datetime import datetime


def is_file(filename):
    """
    判断是否为文件
    :param filename:文件
    :return:文件不存在，不是文件，大小为0，返回 None
    """
    if not os.path.exists(filename):
        print(filename, ' 没有找到')
        return None
    if not os.path.isfile(filename):
        print(filename, ' 不是文件')
        return None
    if os.path.getsize(filename) == 0:
        print(filename, ' 大小为 0')
        return None

    return filename


def make_dir(_path):
    """
    检查文件目录是否存在，如果不存在则创建。
    """
    if not os.path.exists(_path):
        os.makedirs(_path)


def exist_or_makedir(in_dir):
    """
    检查文件所在的父级文件夹是否存在，如果不存在，就创建父级文件夹
    :param in_dir:
    :return: None
    """
    output_dir = os.path.dirname(in_dir)
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
        return output_dir


def get_encoding(fromfile):
    """
    文件编码判断
    :param fromfile: 
    :return: 编码格式
    """
    with open(fromfile, 'rb') as tt:
        ff = tt.readline()
        # 这里试着换成read(5)也可以，但是换成readlines()后报错
        enc = chardet.detect(ff)
        # print(enc['encoding'])
        return (enc['encoding'])


def validate_title(title):
    """
    去除文件名中的非法字符 (Windows)
    :param title:
    :return:
    """
    rstr = r"[\/\\\:\*\?\"\<\>\|]"  # '/ \ : * ? " < > |'
    # 替换为空格，也可以替换为“_”
    new_title = re.sub(rstr, " ", title)
    return new_title


def replace_invalid_filename_char(filename, replaced_char='_'):
    """
    替换文件名中无效的字符。 默认用'_'替换。

    :param filename:  要替换的文件名
    :param replaced_char:  替换的字符
    :return:
    """
    invalid_characaters = '\\/:*?"<>|'
    for c in invalid_characaters:
        filename = filename.replace(c, replaced_char)

    return filename


def get_file_md5(filename):
    """
    计算文件的 MD5
    :param filename:
    :return:
    """

    if not is_file(filename):
        return None

    md5_ = hashlib.md5()
    with open(filename, 'rb') as f:
        while True:
            data = f.read(4096)
            if not data:
                break
            md5_.update(data)
    return md5_.hexdigest()


def get_text_md5(text):
    """
    计算文件的 MD5
    :param filename:
    :return:
    """
    return hashlib.md5(text.encode('utf-8')).hexdigest()


def from_this_dir(filename):
    """
    获取运行模块所在路径的全路径
    """
    return os.path.join(os.path.dirname(os.path.dirname(__file__)), filename)


def from_this_dir_2(filename):
    """
    获取本模块所在路径的全路径
    """
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), filename)


def get_file_line_info():
    """ 获取当前时间，文件路径，所在行数"""
    try:
        raise Exception
    except:
        f = sys.exc_info()[2].tb_frame.f_back
    # print(dir(f))
    return '%s, File "%s", line %s ' % (
        str(datetime.now()), f.f_code.co_filename, str(f.f_lineno)
    )
    # , f.f_code.co_name(<module>)


def file_string_replace(file, old, new, count=None):
    """
    按行替换文件内容
    """

    counts = 0
    with open(file, "r", encoding="utf-8") as f:
        # readlines以列表的形式将文件读出
        lines = f.readlines()
    with open(file, "w", encoding="utf-8") as f_w:
        # 定义一个数字，用来记录在读取文件时在列表中的位置
        n = 0
        # 默认选项，只替换第一次匹配到的行中的字符串
        for line in lines:
            if old in line:
                counts += 1
                line = line.replace(old, new)
            f_w.write(line)

    if counts:
        print("在 %s 中替换了 %s 为 %s，总共 %s个" % (file, old, new, counts))
    else:
        print("没有内容替换")


def exists_file(_file):
    """
    判断文件是否存在，如果存在就返回重命名的文件，不存在就直接返回
    :param _file:
    :return:
    """
    if os.path.exists(_file):
        while True:
            f_name, f_ext = os.path.splitext(_file)
            _file2 = f_name + '-' + str(random.randint(0, 10000000)) + f_ext
            if _file != _file2:
                break
        return _file2
    else:
        return _file


def exists_file_to_bak(_file):
    """
    如果文件已经存在，拷贝到 bak 文件夹
    """
    if os.path.exists(_file):
        while True:
            file_path, file_name = os.path.split(_file)
            f_name, f_ext = os.path.splitext(file_name)
            _file2 = os.path.join(file_path, 'bak', f_name + '-' + datetime.now().strftime('%Y%m%d%H%M%S') + f_ext)
            if _file != _file2:
                break
        make_dir(os.path.dirname(_file2))
        shutil.move(_file, _file2)
        # print(shutil.move(_file, _file2))
        # print((_file, _file2))


def get_name(path):
    """
    获取路径中最后的文件名 不包括后缀名
    """
    # os.path.split 分离路径的目录名和文件名(dirname(), basename()) 元组，后一部分总是最后级别的目录或文件名
    _, name = os.path.basename(path)
    # 分离文件名和扩展名， 返回(filename,extension)元组，没有扩展名，扩展名返回空
    ret_name, _ = os.path.splitext(name)
    return ret_name


def list_dir(file_dir):
    """
    获取文件夹下的文件列表
    跳过目录、文件名前缀是.的文件

    :param file_dir:
    :return: 返回文件列表的生成器
    """
    if os.path.isdir(file_dir):
        for name in os.listdir(file_dir):
            file = os.path.join(file_dir, name)

            # 跳过目录
            if os.path.isdir(file):
                continue

            # 跳过文件名前缀是.的文件
            if name.startswith('.'):
                continue

            # print(file)
            yield file
    else:
        raise NotADirectoryError(file_dir)


def from_dir_func(dir_path, func, endswith='', *args, **kwargs):
    """
    处理目录中的所有文件，跳过前缀是.的文件名

    # 扩展阅读
    from functools import partial
    基于一个函数创建一个新的可调用对象，把原函数的某些参数固定。
    使用这个函数可以把接受一个或多个参数的函数改编成需要回调的 API，这样参数更少。
    new_func = partial(test, 22222)
    """

    fileok = 0
    fileno = 0
    if os.path.isdir(dir_path):
        print('\n处理路径：\n%s\n' % (dir_path))
        for dirpath, dirnames, filenames in os.walk(dir_path):
            for _filename in filenames:
                if _filename.startswith('.'):
                    continue

                _file = os.path.join(dirpath, _filename)
                if endswith:
                    if _file.endswith(endswith):
                        fileok += 1
                        # print(_file)
                        func(_file, *args, **kwargs)
                    else:
                        fileno += 1
                        # print('忽略 --------------------', _file)
                else:
                    fileok += 1
                    func(_file)

        print(' ----------- 处理 %s 个文件（处理条件：%s） ----------- 忽略 %s 个文件 ----------- ' % (fileok, endswith, fileno))
    else:
        print('请输入文件路径！')


def get_walk_files(dir_path, endswith=''):
    """
    获取文件夹包括子文件夹里面的文件列表（生成器）
    """
    if os.path.isdir(dir_path):
        print('\n处理路径：\n%s\n' % (dir_path))
        for dirpath, dirnames, filenames in os.walk(dir_path):
            for _filename in filenames:
                if _filename.startswith('.'):
                    continue

                _file = os.path.join(dirpath, _filename)
                if endswith:
                    if _file.endswith(endswith):
                        yield _file
                else:
                    yield _file
    else:
        return None


def get_dir_files(path, ext=''):
    """
    获取文件夹里面，指定后缀名的文件列表（生成器）
    例子：
    get_dir_files(r".",'py')
    """
    # files = []
    for _file in os.listdir(path):
        if _file.endswith(ext):
            file_path = os.path.join(path, _file)
            if os.path.isfile(file_path):
                # files.append(file_path)
                yield file_path
                # print(_file)
    # return files


def save_file(s, file, mode='w', encoding='utf-8'):
    """保存字符内容到文件"""
    with open(file, mode, encoding=encoding) as fp:
        fp.write(s)


def doc():
    """
    打印模块说明文档
    """
    doc_text = """
    # 默认字典的例子
    from collections import defaultdict
    def get_counts2(sequence):
        counts = defaultdict(int) # 所有的值均会被初始化为0
        for x in sequence:
            counts[x] += 1
        return counts
    
    """
    doc_text += '\n'
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=is_file)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=make_dir)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=exist_or_makedir)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_encoding)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=validate_title)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=replace_invalid_filename_char)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_file_md5)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_text_md5)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=from_this_dir)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=from_this_dir_2)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_file_line_info)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=file_string_replace)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=exists_file)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=exists_file_to_bak)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=get_name)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=list_dir)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=from_dir_func)
    doc_text += '{fun.__name__}{fun.__doc__}\n'.format(fun=save_file)
    print(doc_text)


if __name__ == '__main__':
    # 记录运行时间 --------------------------------------------------
    from time import time, sleep

    start_time = t1 = time()

    doc()

    # 性能测试
    # from timeit import timeit
    # print(timeit('validate_title("\\fasdf/f:*?dasfdddddddddddddddddddddda<fasf>|")', 'from file import validate_title', number=1000000))
    # replace_invalid_filename_char 稍微快一点
    # print(timeit('replace_invalid_filename_char("\\fasdf/f:*?dasfdddddddddddddddddddddda<fasf>|")', 'from file import replace_invalid_filename_char', number=1000000))

    print('运行时间 %.2f 秒' % (time() - start_time))
